﻿using sfs2x.extension.test.serialization.model;
using Sfs2X;
using Sfs2X.Core;
using Sfs2X.Entities.Data;
using Sfs2X.Protocol.Serialization;
using Sfs2X.Requests;
using Sfs2X.Util;
using System;
using System.Reflection;

namespace RpgExtensionClient
{
    class Program
    {
        private SmartFox sfs;

        public Program() 
        {
            DefaultSFSDataSerializer.RunningAssembly = Assembly.GetExecutingAssembly();
            start();
        }

        private void start()
        {
            var cfg = new ConfigData();
            cfg.Host = "192.168.1.252";
            cfg.Port = 9933;
            cfg.Zone = "BasicExamples";
            cfg.Debug = false;

            sfs = new SmartFox();
            sfs.ThreadSafeMode = false;
            sfs.AddEventListener(SFSEvent.CONNECTION, OnConnection);
            sfs.AddEventListener(SFSEvent.CONNECTION_LOST, OnConnectionLost);
            sfs.AddEventListener(SFSEvent.LOGIN, OnLogin);
            sfs.AddEventListener(SFSEvent.EXTENSION_RESPONSE, OnExtensionResponse);

            Console.WriteLine("Connecting...");
            sfs.Connect(cfg);
        }

        public void OnConnection(BaseEvent evt)
        {
            var success = (bool) evt.Params["success"];

            if (success)
            {
                Console.WriteLine("Connection success");
                sfs.Send(new LoginRequest(""));
            }
        }

        public void OnLogin(BaseEvent evt)
        {
            Console.WriteLine("Logged in as: " + sfs.MySelf.Name);

            sfs.Send(new ExtensionRequest("getModel", new SFSObject()));
        }

        public void OnConnectionLost(BaseEvent evt)
        {
            Console.WriteLine("Connection Lost!");
        }

        public void OnExtensionResponse(BaseEvent evt)
        {
            var res = (ISFSObject) evt.Params["params"];
            Console.WriteLine("Extension response!");

            dumpModel(res);
        }

        private void dumpModel(ISFSObject data)
        {
            var characters = data.GetSFSArray("knights");
            for (int i = 0; i < characters.Size(); i++)
            {
                RpgCharacter knight = (RpgCharacter) characters.GetClass(i);
                Console.WriteLine(knight);
            }   
        }

        // ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

        static void Main(string[] args)
        {
            new Program();
            
            // Wait until a key is pressed
            Console.ReadKey();
        }
    }
}
