package sfs2x.extension.test.serialization;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import sfs2x.extension.test.serialization.model.CharacterProperty;
import sfs2x.extension.test.serialization.model.DeathRaySpell;
import sfs2x.extension.test.serialization.model.Item;
import sfs2x.extension.test.serialization.model.KnifeItem;
import sfs2x.extension.test.serialization.model.KnightSkin;
import sfs2x.extension.test.serialization.model.RisingFlamesSpell;
import sfs2x.extension.test.serialization.model.RpgCharacter;
import sfs2x.extension.test.serialization.model.ShieldItem;
import sfs2x.extension.test.serialization.model.SolarVortexSpell;
import sfs2x.extension.test.serialization.model.SorcererSkin;
import sfs2x.extension.test.serialization.model.Spell;
import sfs2x.extension.test.serialization.model.SpiderSwarmSpell;
import sfs2x.extension.test.serialization.model.SwordItem;
import sfs2x.extension.test.serialization.model.WaterFloodSpell;

import com.smartfoxserver.v2.annotations.Instantiation;
import com.smartfoxserver.v2.annotations.Instantiation.InstantiationMode;
import com.smartfoxserver.v2.entities.User;
import com.smartfoxserver.v2.entities.data.ISFSArray;
import com.smartfoxserver.v2.entities.data.ISFSObject;
import com.smartfoxserver.v2.entities.data.SFSArray;
import com.smartfoxserver.v2.entities.data.SFSObject;
import com.smartfoxserver.v2.extensions.BaseClientRequestHandler;

@Instantiation(InstantiationMode.SINGLE_INSTANCE)
public class GetModelHandler extends BaseClientRequestHandler
{
	private final ISFSObject model;
	
	public GetModelHandler()
    {
		this.model = generateModel();
    }
	
	@Override
	public void handleClientRequest(User sender, ISFSObject params)
	{
		trace("Sending model to requester: " + sender);
		
		// We just send the data model as SFSObject
	    send("getModel", model, sender);
	}
	
	/*
	 * Generate the data model 
	 */
	private ISFSObject generateModel()
	{
		// --- Character #1 -------------------------------------------------------------------------------------------------------
		RpgCharacter knightOfTheRoundTable1 = new RpgCharacter("Sigfried", "knight", new KnightSkin("roundTableKnight", "knightSkin"));
		
		List<Item> items1 = new ArrayList<Item>();
		items1.add(new SwordItem("longSword", 500));
		items1.add(new SwordItem("shortSword", 200));
		
		Map<String, CharacterProperty> props1 = new HashMap<String, CharacterProperty>();
		props1.put("strength", new CharacterProperty(60, 100, "str"));
		props1.put("endurance", new CharacterProperty(70, 100, "end"));
		props1.put("combatSkill", new CharacterProperty(55, 100, "csk"));
		
		Map<String, Spell> spells1 = new HashMap<String, Spell>();
		spells1.put("swarm", new SpiderSwarmSpell("spss", 3));
		
		knightOfTheRoundTable1.setInventory(items1);
		knightOfTheRoundTable1.setProperties(props1);
		knightOfTheRoundTable1.setSpells(spells1);
		
		// --- Character #2 -------------------------------------------------------------------------------------------------------
		RpgCharacter knightOfTheRoundTable2 = new RpgCharacter("Tristan", "knight", new KnightSkin("roundTableKnight", "knightSkin"));
		
		List<Item> items2 = new ArrayList<Item>();
		items2.add(new SwordItem("DragonSword", 1500));
		items2.add(new KnifeItem("IceKnife", 300));
		items2.add(new ShieldItem("IronShield", 150));
		
		Map<String, CharacterProperty> props2 = new HashMap<String, CharacterProperty>();
		props2.put("strength", new CharacterProperty(80, 100, "str"));
		props2.put("endurance", new CharacterProperty(50, 100, "end"));
		props2.put("combatSkill", new CharacterProperty(75, 100, "csk"));
		
		knightOfTheRoundTable2.setInventory(items2);
		knightOfTheRoundTable2.setProperties(props2);
		
		// --- Character #3 -------------------------------------------------------------------------------------------------------
		RpgCharacter knightOfTheRoundTable3 = new RpgCharacter("Hayden", "sorcerer", new SorcererSkin("roundTableSorcerer", "socererSkin"));
		
		List<Item> items3 = new ArrayList<Item>();
		items3.add(new KnifeItem("DiamondKnife", 600));
		
		Map<String, CharacterProperty> props3 = new HashMap<String, CharacterProperty>();
		props3.put("strength", new CharacterProperty(50, 100, "str"));
		props3.put("endurance", new CharacterProperty(50, 100, "end"));
		props3.put("combatSkill", new CharacterProperty(55, 100, "csk"));
		
		Map<String, Spell> spells3 = new HashMap<String, Spell>();
		spells3.put("vortex", new SolarVortexSpell("svs", 30));
		spells3.put("flames", new RisingFlamesSpell("rfs", 20));
		spells3.put("swarm", new SpiderSwarmSpell("sps", 15));
		spells3.put("flood", new WaterFloodSpell("wfs", 3));
		
		knightOfTheRoundTable3.setInventory(items3);
		knightOfTheRoundTable3.setProperties(props3);
		knightOfTheRoundTable3.setSpells(spells3);
		
		
		// --- Prepare SFSOBJECT --------------------------------------------------------------------
		ISFSObject sfsObj = SFSObject.newInstance();
		
		ISFSArray knightsArray = SFSArray.newInstance();
		knightsArray.addClass(knightOfTheRoundTable1);
		knightsArray.addClass(knightOfTheRoundTable2);
		knightsArray.addClass(knightOfTheRoundTable3);
		
		sfsObj.putSFSArray("knights", knightsArray);
		sfsObj.putClass("knife", new KnifeItem("MortalKnife", 3000));
		sfsObj.putClass("spell", new DeathRaySpell("drs", 90));
		sfsObj.putBool("isRpg", true);
		sfsObj.putIntArray("randomValues", Arrays.asList(new Integer[] {100,45,62,51,7,13,161}));
		
		return sfsObj;
	}
}
